//
// Copyright (c) 2002-2022 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#ifndef __WZCPP_ALGORITHM_APPEND_H__
#define __WZCPP_ALGORITHM_APPEND_H__

#include <iterator>
#include <utility>
#include <vector>
#include <map>

namespace wzcpp
{

/*! Allows a vector to be appended to another of the src is an rvalue the content is moved */

template<typename T>
typename std::vector<T>::iterator append(const std::vector<T>& src, std::vector<T>& dest)
{
	typename std::vector<T>::iterator result;

	if (dest.empty())
	{
		dest   = src;
		result = std::begin(dest);
	}
	else
	{
		result = dest.insert(std::end(dest), std::cbegin(src), std::cend(src));
	}

	return result;
}

template<typename T>
typename std::vector<T>::iterator append(std::vector<T>&& src, std::vector<T>& dest)
{
	typename std::vector<T>::iterator result;

	if (dest.empty())
	{
		dest   = std::move(src);
		result = std::begin(dest);
	}
	else
	{
		result = dest.insert(std::end(dest),
		                     std::make_move_iterator(std::begin(src)),
		                     std::make_move_iterator(std::end(src)));
	}

	src.clear();
	src.shrink_to_fit();

	return result;
}

template<class TKey, class TValue>
void append(std::map<TKey, TValue>&& src, std::map<TKey, TValue>& dest)
{
	dest.merge(src);
	src.clear();
}

}  // end namespace wzcpp
#endif  // __WZCPP_ALGORITHM_APPEND_H__