//
// Copyright (c) 2002-2023 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#ifndef __WZCPP_UTILS_MEMORY_H__
#define __WZCPP_UTILS_MEMORY_H__
#include <memory>

namespace wzcpp
{
template<typename T, typename... Args>
std::unique_ptr<T> make_unique(Args&&... args)
{
// If the cpp version is greater than c++11
// use the make_unique from std library
// otherwise use our own implementation.
#if (__cplusplus > 201103L)
	return std::make_unique<T>(std::forward<Args>(args)...);
#else
	return std::unique_ptr<T>(new T(std::forward<Args>(args)...));
#endif
}

}  // namespace wzcpp
#endif
