//
// Copyright (c) 2002-2020 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#ifndef __BAOSLIB_CONFIG_H__
#define __BAOSLIB_CONFIG_H__

#include "baos/BaosLib_export.h"


/// @cond DOCUMENT_STL
#include <cstdint>
#include <string>
#include <vector>
/// @endcond


/*!
    @file
    @brief Global types and configuration for the whole SDK
*/

/*******************************************
** Platform specific defines
********************************************/

//
// Define WIN32_LEAN_AND_MEAN for windows to avoid include problems
// with duplicate defines.
//
// From msdn: https://learn.microsoft.com/en-us/windows/win32/winsock/creating-a-basic-winsock-application
// The Winsock2.h header file internally includes core elements from the Windows.h header file.
// For historical reasons, the Windows.h header defaults to including the Winsock.h header file for Windows Sockets 1.1.
// The declarations in the Winsock.h header file will conflict with the declarations in the Winsock2.h header file
// required by Windows Sockets 2.0. The WIN32_LEAN_AND_MEAN macro prevents the Winsock.h from being included by the
// Windows.h header.
//
// One solution is to include Winsock2.h before include Windows.h but other libraries (e.g. fmt) could include Windows.h
// before we include Winsock2.h in TcpDriver_win.h.
// So we use the secound solution: Define here WIN32_LEAN_AND_MEAN.
//

#ifdef WZPLATFORM_WIN32
#if !defined(WIN32_LEAN_AND_MEAN)
#define WIN32_LEAN_AND_MEAN
#endif
#endif

/*******************************************
** Global types
********************************************/

namespace wz::baos
{

using Buffer = std::vector<std::uint8_t>;           //!< An alias type for a byte buffer
constexpr std::uint16_t MAX_TELEGRAM_LENGTH{1024};  //!< The maximum supported length for one BAOS telegram.

}  // namespace wz::baos

#endif  // __BAOSLIB_CONFIG_H__
