//
// Copyright (c) 2002-2023 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#ifndef __BAOSLIB_UTILITY_H__
#define __BAOSLIB_UTILITY_H__

#include "baos/config.h"
#include "baos/protocol/Defines.h"
#include <cassert>
#include <memory>
#include <string>


/*!
    @file
    @brief Generic utility functions and classes
*/
namespace wz::utility
{

/*!
    @brief Get the high byte of a value

    @tparam T The type from which the high byte should be retrieved.
    @param value The actual value to extract from
    @return std::uint8_t  The high byte or the second byte if T has more than 2 bytes
 */
template<typename T>
std::uint8_t hiByte(T value)
{
	assert(sizeof(T) > 1);
	return static_cast<std::uint8_t>((value >> 8) & 0xff);
};

/*!
    @brief Get the low byte of a value

    @tparam T The type from which the low byte should be retrieved.
    @param value The actual value to extract from
    @return std::uint8_t The low byte
*/
template<typename T>
std::uint8_t lowByte(const T value)
{
	assert(sizeof(T) > 1);
	return static_cast<std::uint8_t>(value & 0xff);
};


extern template std::uint8_t hiByte<std::uint16_t>(std::uint16_t);  // explicit instanziation of template
template<>
std::uint8_t hiByte<wz::baos::protocol::ServerItemId>(wz::baos::protocol::ServerItemId value);


extern template std::uint8_t lowByte<std::uint16_t>(std::uint16_t);  // explicit instanziation of template
template<>
std::uint8_t lowByte<wz::baos::protocol::ServerItemId>(wz::baos::protocol::ServerItemId value);

}  // namespace wz::utility

#endif  // __BAOSLIB_UTILITY_H__
